//
// RT linux Implementation of Logic Scope
// Author: Bill Crum
//         New Mexico Tech
// Email   tobor@nmt.edu
//


#include "logic.h"
#include <qdatetm.h>
#include <qpainter.h>


static unsigned int trigmask;
static unsigned int levelmask;
static unsigned int single_shot;

//
// Sample rate from fifo interval 50 = 50ms 
//


LogicTimer::LogicTimer( QWidget *parent ) : QTimer( parent ), interval( 10 )
{
    initMetaObject();
    start( interval );
}


// The Digital Scope canvas

LogicCanvas::LogicCanvas ( QWidget *parent, const char *name)
  : QFrame( parent, name)
{
  char buffer[5000];
    initMetaObject();

    setMinimumSize( 100 + (2 * BORDER),100 + (2 * BORDER) );
    setMaximumSize( 1000 + (2 * BORDER), 1000 + (2 * BORDER) );

    setSizeIncrement( 1, 1);
  
    clear();
    timer = new LogicTimer(this);

    connect( timer, SIGNAL(timeout()), SLOT(drawSignals()) );
    time_scale = 25;
    voltage_scale = 1;


    if ((dio = open("/dev/rtf9", O_RDONLY)) < 0) {
      fprintf(stderr, "Error opening /dev/rtf9\n");
      exit(1);
    }
    read(dio, buffer, 5000);

  
}


//
// Constructs a DigitalClock widget with a parent and a name.
//

Logic::Logic( QWidget *parent, const char *name )
    : QFrame( parent, name )
{
  initMetaObject();				// initialize meta object

 // Create a combo box

  QLabel *msgLabel1 = new QLabel( this, "msgLabel1" );

  msgLabel1->setText( "Scan Rate:" );
  msgLabel1->setAlignment( AlignLeft|AlignVCenter );
  msgLabel1->setGeometry( 0,0, 65, 25 );
  msgLabel1->move(105,5);

  QComboBox *combo = new QComboBox( this, "comboBox" );
  combo->insertItem( "Continuous" );
  combo->insertItem( "One Shot  " );

  combo->setGeometry( 0, 0, 120, 25 );
  combo->setAutoResize( FALSE );
  combo->move(170,5 );


  QComboBox *combo2 = new QComboBox( this, "comboBox2" );
  combo2->insertItem( "10  ms / Div" );
  combo2->insertItem( " 5  ms / Div" );
  combo2->insertItem( " 2  ms / Div" );
  combo2->insertItem( " 1  ms / Div" );

  combo2->setGeometry( 0, 0, 120, 25 );
  combo2->setAutoResize( FALSE );
  combo2->move(300,5 );


  //  QLabel *msgLabel2 = new QLabel( this, "msgLabel2" );

 // Create a combo box

    pb1 = new QPushButton( "Trigger", this );
    pb1->setToggleButton( FALSE );
    pb1->resize( 120, 25 );
    pb1->move( 550,5 );

    pb = new QPushButton( "Pause", this );
    pb->setToggleButton( TRUE );
    pb->resize( 120, 25 );
    pb->move( 671,5 );

    logic = new LogicCanvas(this);
    logic->move( 0,30 );
    connect( combo, SIGNAL(activated(int)), logic, SLOT(set_voltage_scale(int)) );
    connect( combo2, SIGNAL(activated(int)), logic, SLOT(set_time_scale(int)) );
    connect( pb, SIGNAL(toggled(bool)), logic, SLOT(hold()));
    connect( pb1, SIGNAL(clicked()), logic, SLOT(set_trigger()));

    QSize s;
    s = logic->minimumSize();
    setMinimumSize( 800,300);
    s = logic->maximumSize();
    setMaximumSize( 800,300 );
    s = logic->sizeIncrement();
    setSizeIncrement( s.width(), s.height() );

    logic->setFrameStyle( QFrame::Panel | QFrame::Sunken );
    logic->move(105,30 );
    logic->resize(691, 268 );
    resize( 800, 300 );

    QButtonGroup *bg = new QButtonGroup( this, "checkGroup" );
    bg->setTitle( "Trigger" );
    cb[0] = new QCheckBox( bg );
    cb[0]->setText( "Ch1" );
    cb[0]->setGeometry( 5,20, 40,25 );
    cb[1] = new QCheckBox( bg );
    cb[1]->setText( "Ch2" );
    cb[1]->setGeometry( 5,52, 40,25 );
    cb[2] = new QCheckBox( bg );
    cb[2]->setText( "Ch3" );
    cb[2]->setGeometry( 5,84, 40,25 );
    cb[3] = new QCheckBox( bg );
    cb[3]->setText( "Ch4" );
    cb[3]->setGeometry( 5,116, 40,25 );
    cb[4] = new QCheckBox( bg );
    cb[4]->setText( "Ch5" );
    cb[4]->setGeometry( 5,148, 40,25 );
    cb[5] = new QCheckBox( bg );
    cb[5]->setText( "Ch6" );
    cb[5]->setGeometry( 5,180, 40,25 );
    cb[6] = new QCheckBox( bg );
    cb[6]->setText( "Ch7" );
    cb[6]->setGeometry( 5,212, 40,25 );
    cb[7] = new QCheckBox( bg );
    cb[7]->setText( "Ch8" );
    cb[7]->setGeometry( 5,244, 40,25 );

    cb[8] = new QCheckBox( bg );
    cb[8]->setText( "X" );
    cb[8]->setGeometry( 60,20, 40,25 );
    cb[9] = new QCheckBox( bg );
    cb[9]->setText( "X" );
    cb[9]->setGeometry( 60,52, 40,25 );
    cb[10] = new QCheckBox( bg );
    cb[10]->setText( "X" );
    cb[10]->setGeometry( 60,84, 40,25 );
    cb[11] = new QCheckBox( bg );
    cb[11]->setText( "X" );
    cb[11]->setGeometry( 60,116, 40,25 );
    cb[12] = new QCheckBox( bg );
    cb[12]->setText( "X" );
    cb[12]->setGeometry( 60,148, 40,25 );
    cb[13] = new QCheckBox( bg );
    cb[13]->setText( "X" );
    cb[13]->setGeometry( 60,180, 40,25 );
    cb[14] = new QCheckBox( bg );
    cb[14]->setText( "X" );
    cb[14]->setGeometry( 60,212, 40,25 );
    cb[15] = new QCheckBox( bg );
    cb[15]->setText( "X" );
    cb[15]->setGeometry( 60,244, 40,25 );
    trigmask = 0;
    bg->setGeometry( 4,25, 100,270 );
    bg->setFrameStyle(NoFrame);
    connect( bg, SIGNAL(clicked(int)),  SLOT(checkBoxClicked(int)) );


    showingColon = FALSE;
    setFrameStyle( QFrame::Box | QFrame::Raised );

    setLineWidth( 2 );				// set frame line width
    // showTime();					// display the current time
    logic->show();
    //    showDateTimer = -1;				// not showing date
}

//
// Handles the trigger mask that is created from the 16 buttons that 
// enable or disable level triggering.
// Left button per channel enable/disable triggering
// Right button per channel selects look for 0 (low) or 1 (high) level
//

void Logic::checkBoxClicked( int id )
{
  unsigned char bytemask;
  switch (id)
    {
    case 0:
    case 1:
    case 2:
    case 3:
    case 4:
    case 5:
    case 6:
    case 7:
      bytemask = 0x01;
      bytemask <<= id;
      if ( cb[id]->isChecked() )
	{
	  if (cb[id+8]->isChecked() )
	    {
	      cb[id+8]->setText( "1" );
	      levelmask |= bytemask;
	    }
	  else
	    {
	      cb[id+8]->setText( "0" );
	    }
	  trigmask |= bytemask;
	}

      else
	{
	  cb[id+8]->setText( "X" );
	  trigmask &= ~bytemask;
	  levelmask &= ~bytemask;
	}
	      
      break;

    case 8:
    case 9:
    case 10:
    case 11:
    case 12:
    case 13:
    case 14:
    case 15:
      bytemask =   0x01;
      bytemask <<= (id-8);
      if ( cb[id-8]->isChecked() )
	{
	  if (cb[id]->isChecked() )
	    {
	      cb[id]->setText( "1" );
	      levelmask |= bytemask;
	    }
	  else
	    {
	      cb[id]->setText( "0" );
	      levelmask &= ~bytemask;
	    }
	}
      else
	{
	  cb[id]->setText( "X" );
	}
	      
      break;
    }
}


// Stops update to canvas.

void LogicCanvas::hold()
{
  if (holdDisplay)
    holdDisplay = 0;
  else
    holdDisplay = 1;
}




void LogicCanvas::set_voltage_scale(int scale)
{
  if (scale)
    {
      wait_for_trigger = 1;
      single_shot = 1;
    }
  else
    {
      wait_for_trigger = 0;
      single_shot = 0;
    }
}

void LogicCanvas::set_time_scale(int scale)
{
  switch (scale)
    {
    case 0:
      time_scale = 25; // 10 ms
      break;
    case 1:
      time_scale = 50; // 5 ms / div
      break;
    case 2:
      time_scale = 100; // 2 ms / div
      break;
    case 3:
      time_scale = 250; // 1 ms / div
      break;
    }      

}

//
// Sets the trigger if in one shot mode.
//

void LogicCanvas::set_trigger()
{

  wait_for_trigger  = 1;
}


//
// Checks if inbyte meets criteria for triggering
//

int LogicCanvas::trigger(int inbyte)
{
  int loop;
  int local_inbyte;
  unsigned int local_trigmask;
  unsigned int local_levmask;
  
  local_trigmask = trigmask;
  local_levmask = levelmask;

  local_inbyte = inbyte;
  if (single_shot && !wait_for_trigger)
    return 0;
  if (wait_for_trigger || trigmask)
    {
      for (loop = 0; loop < 8; loop++)
	{
	  if (local_trigmask & 0x01) // do we check this channel
	    { // yes, check if correct level
	      if ( local_levmask & 0x01 ) // looking for 1
		{
		  if (!(local_inbyte & 0x01))
		    return 0; // already found nonmatch, no trigger
		}
	      else
		{
		  if (local_inbyte & 0x01) // nonmatch, wanted 0
		    return 0;
		}
	    }
	  local_levmask >>= 1;
	  local_inbyte >>= 1;
	  local_trigmask >>= 1;
	}
      if (single_shot)
	wait_for_trigger = 0;
      clear();
      return 1; // if here all matched
    }
  else
    {
      return 1;
    }
}

//
// Updates the Canvas with the signals. Gets data  from real time fifo 3
//

void LogicCanvas::drawSignals()
{
  int new_scale;
  int lloop;

  if (x > 1000)
    {
      if (!holdDisplay && (!single_shot && !trigmask))
        {
          x = 0;
          oldx = x;
          clear();
        }
      if (!holdDisplay && (trigmask || single_shot))
	{
	  x = 0;
	  oldx = x;
	}
    }


  QPainter paint;
  new_scale = voltage_scale;
  /*  while (rt_fifo_get(DIO_FIFO_IN,(char *) &inbyte,2) > 0)*/

  for (lloop = 0; lloop < 25; lloop++)
    {
      read(dio, &inbyte, 1);
  
      oldx = x;

      if (x < 1010)
	{
	  z = (unsigned int) inbyte;
	  if ( ((x == 0) && trigger(z)) || (x != 0))
	    {
	      paint.begin(this);
	      paint.setWindow( 0,0,1000,1650 );
	      paint.setPen(blue);
	      x  += time_scale;
	      zz = z;
	      for (i = 1; i < 9; i++)
		{
		  y = 200 * i;
		  if (z & 0x01)
		    {
		      if (!(oldz & 0x01))
			paint.drawLine(oldx,y,oldx,y - 150);
		      paint.drawLine(oldx,y - 150 ,x,y - 150);
		    }
		  else
		    {
		      //		  y -= 150;
		      if (oldz & 0x01)
			paint.drawLine(oldx,y-150,oldx,y);
		      paint.drawLine(oldx,y,x,y);
		    }
		  z >>= 1;
		  oldz >>= 1;
		}
	      oldz = zz;

	      paint.end();	  
	    }
	}
    }
}

// There must be an easier way


void LogicCanvas::clear()
{
      repaint();

      QPainter paint;
      paint.begin( this ); 
      paint.setWindow( 0,0,400,1650 );


      paint.setPen(green);

      paint.drawLine(0,200,396,200);
      paint.drawLine(0,400,396,400);
      paint.drawLine(0,600,396,600);
      paint.drawLine(0,800,396,800);
      paint.drawLine(0,1000,396,1000);
      paint.drawLine(0,1200,396,1200);
      paint.drawLine(0,1400,396,1400);
      paint.drawLine(0,1600,396,1600);

      paint.drawLine(20,0,20,1650);
      paint.drawLine(40,0,40,1650);
      paint.drawLine(60,0,60,1650);
      paint.drawLine(80,0,80,1650);
      paint.drawLine(100,0,100,1650);
      paint.drawLine(120,0,120,1650);
      paint.drawLine(140,0,140,1650);
      paint.drawLine(160,0,160,1650);
      paint.drawLine(180,0,180,1650);
      paint.drawLine(200,0,200,1650);
      paint.drawLine(220,0,220,1650);
      paint.drawLine(240,0,240,1650);
      paint.drawLine(260,0,260,1650);
      paint.drawLine(280,0,280,1650);
      paint.drawLine(300,0,300,1650);
      paint.drawLine(320,0,320,1650);
      paint.drawLine(340,0,340,1650);
      paint.drawLine(360,0,360,1650);
      paint.drawLine(380,0,380,1650);

      paint.end();

}






